@@ -18,7 +18,6 @@
 
 #include "analitzatest.h"
 #include "analyzer.h"
-#include <qtest_kde.h>
 #include <cmath>
 
 #include "apply.h"
@@ -28,9 +27,9 @@
 #include "value.h"
 #include <variable.h>
 #include <analitzautils.h>
+#include <QtTest/QTest>
 //#include <operations.h>
 
-
 using namespace std;
 using Analitza::Cn;
 using Analitza::Ci;
@@ -40,7 +39,16 @@ using Analitza::Operator;
 using Analitza::Container;
 using Analitza::Expression;
 
-QTEST_KDEMAIN_CORE( AnalitzaTest )
+QTEST_MAIN( AnalitzaTest )
+
+namespace QTest {
+
+template <> char *toString(const Analitza::Cn &cn)
+{ return qstrdup(QString::fromLatin1("Cn(%1)").arg(cn.toString()).toLatin1().constData()); }
+
+}
+
+Q_DECLARE_METATYPE(Cn);
 
 AnalitzaTest::AnalitzaTest(QObject *parent)
  : QObject(parent)
@@ -62,53 +70,57 @@ void AnalitzaTest::cleanupTestCase()
 void AnalitzaTest::testTrivialCalculate_data()
 {
 	QTest::addColumn<QString>("expression");
-	QTest::addColumn<double>("result");
+	QTest::addColumn<Cn>("result");
 
-	QTest::newRow("a value") << "2" << 2.;
-	QTest::newRow("val.e0") << "12.0e-02" << 12e-2;
-	QTest::newRow("vale") << "12e-2" << 12e-2;
-	QTest::newRow("val") << "12e2" << 12e2;
+	QTest::newRow("a value") << "2" << Cn(2.);
+	QTest::newRow("val.e0") << "12.0e-02" << Cn(12e-2);
+	QTest::newRow("vale") << "12e-2" << Cn(12e-2);
+	QTest::newRow("val") << "12e2" << Cn(12e2);
 	
-	QTest::newRow("simple addition") << "2+2" << 4.;
-	QTest::newRow("simple power") << "2**99" << pow(2., 99.);
-	QTest::newRow("simple multiplication") << "3*3" << 9.;
-	QTest::newRow("sinus") << "sin(3*3)" << sin(9.);
-	QTest::newRow("declare") << "x:=3" << 3.;
-	QTest::newRow("sum") << "sum(x : x=1..99)" << 4950.;
-	QTest::newRow("diff") << "(diff(x:x))(1)" << 1.;
-	QTest::newRow("diffz") <<"(diff(z:z))(1)" << 1.;
+	QTest::newRow("simple addition") << "2+2" << Cn(4.);
+	QTest::newRow("simple power") << "2**99" << Cn(pow(2., 99.));
+	QTest::newRow("simple multiplication") << "3*3" << Cn(9.);
+	QTest::newRow("sinus") << "sin(3*3)" << Cn(sin(9.));
+	QTest::newRow("declare") << "x:=3" << Cn(3.);
+	QTest::newRow("sum") << "sum(x : x=1..99)" << Cn(4950.);
+	QTest::newRow("diff") << "(diff(x:x))(1)" << Cn(1.);
+	QTest::newRow("diffz") <<"(diff(z:z))(1)" << Cn(1.);
 	
-	QTest::newRow("product") << "product(n : n=1..5)" << 120.;
-	QTest::newRow("factorial") << "factorial(5)" << 120.;
+	QTest::newRow("product") << "product(n : n=1..5)" << Cn(120.);
+	QTest::newRow("factorial") << "factorial(5)" << Cn(120.);
 	
-	QTest::newRow("simple piecewise") << "piecewise { pi=0? 3, pi=pi?33 }" << 33.;
-	QTest::newRow("simple piecewise with otherwise") << "piecewise { pi=0? 3, ?33 }" << 33.;
-	QTest::newRow("boolean and") << "and(true,false)" << 0.;
-	QTest::newRow("boolean or") << "or(false,true)" << 1.;
-	QTest::newRow("boolean not") << "not(false)" << 1.;
-	QTest::newRow("lambda")  << "(x->x+2)(2)" << 4.;
-	QTest::newRow("lambda2") << "(x->3*x^2)(1)" << 3.;
-	QTest::newRow("lambda3") << "(x->x*sum(t:t=0..3))(2)" << 12.;
-	QTest::newRow("imaginarypow") << "(-4)^(1/4)" << 1.;
-	QTest::newRow("imaginaryroot") << "root(-4, 4)" << 1.;
+	QTest::newRow("simple piecewise") << "piecewise { pi=0? 3, pi=pi?33 }" << Cn(33.);
+	QTest::newRow("simple piecewise with otherwise") << "piecewise { pi=0? 3, ?33 }" << Cn(33.);
+	QTest::newRow("boolean and") << "and(true,false)" << Cn(false);
+	QTest::newRow("boolean or") << "or(false,true)" << Cn(true);
+	QTest::newRow("boolean not") << "not(false)" << Cn(true);
+	QTest::newRow("lambda")  << "(x->x+2)(2)" << Cn(4.);
+	QTest::newRow("lambda2") << "(x->3*x^2)(1)" << Cn(3.);
+	QTest::newRow("lambda3") << "(x->x*sum(t:t=0..3))(2)" << Cn(12.);
+	QTest::newRow("imaginarypow") << "(-4)^(1/4)" << Cn(1.);
+	QTest::newRow("imaginaryroot") << "root(-4, 4)" << Cn(1.);
 
 	//comprehension
-	QTest::newRow("sum.2bvars") << "sum(x*y : (x, y)=1..3)" << 36.;
-	QTest::newRow("sum.list") << "sum(x : x@list{1,5,44})" << 50.;
+	QTest::newRow("sum.2bvars") << "sum(x*y : (x, y)=1..3)" << Cn(36.);
+	QTest::newRow("sum.list") << "sum(x : x@list{1,5,44})" << Cn(50.);
 
-	QTest::newRow("sum.sum") << "sum(sum(x : x=0..i) : i=0..10)" << 220.;
+	QTest::newRow("sum.sum") << "sum(sum(x : x=0..i) : i=0..10)" << Cn(220.);
 
-	QTest::newRow("exists") << "exists(x : x@list{true,true,false})" << 1.;
-	QTest::newRow("forall") << "forall(x : x@list{true,true,false})" << 0.;
+	QTest::newRow("exists") << "exists(x : x@list{true,true,false})" << Cn(true);
+	QTest::newRow("forall") << "forall(x : x@list{true,true,false})" << Cn(false);
 // 	QTest::newRow("emptysum") << "sum(x : x@list{})" << 0.;
 	
-	QTest::newRow("lambdacall") << "f:=x->f(x)" << 0.;
+	QTest::newRow("lambdacall") << "f:=x->f(x)" << Cn(0.);
+	QTest::newRow("cpx1") << "i" << Cn(0, 1);
+	QTest::newRow("cpx2") << "i*i" << Cn(-1);
+	QTest::newRow("cpx3") << "2+i*i" << Cn(1);
+	QTest::newRow("complex number") << "3+4*(5-6*i)" << Cn(23, -24);
 }
 
 void AnalitzaTest::testTrivialCalculate()
 {
 	QFETCH(QString, expression);
-	QFETCH(double, result);
+	QFETCH(Cn, result);
 	Expression e(expression, false);
 	if(!e.isCorrect()) qDebug() << "error: " << e.error();
 	QCOMPARE(e.isCorrect(), true);
@@ -117,12 +129,12 @@ void AnalitzaTest::testTrivialCalculate()
 	
 	if(!a->isCorrect()) qDebug() << "error: " << a->errors();
 	QVERIFY(a->isCorrect());
-	QCOMPARE(a->evaluate().toReal().value(), result);
+	QCOMPARE(a->evaluate().toReal(), result);
 	QVERIFY(a->isCorrect());
 	Expression ee=a->calculate();
 	if(!a->isCorrect()) qDebug() << "error: " << a->errors();
 	QVERIFY(a->isCorrect());
-	QCOMPARE(ee.toReal().value(), result);
+	QCOMPARE(ee.toReal(), result);
 	QVERIFY(a->isCorrect());
 }
 
@@ -130,8 +142,25 @@ void AnalitzaTest::testTrivialEvaluate_data()
 {
 	QTest::addColumn<QString>("expression");
 	QTest::addColumn<QString>("result");
-
+	
 	QTest::newRow("simple value") << "2" << "2";
+	QTest::newRow("complex") << "i*5" << "5*i";
+	QTest::newRow("simple complex value") << "6*(2+i)" << "12+6*i";
+	QTest::newRow("complex irreductibility") << "i" << "i";
+	QTest::newRow("from complex value") << "i*i" << "-1";
+	QTest::newRow("from power complex") << "power(i, 2)" << "-1";
+	QTest::newRow("sin complex") << "sin(i)" << "1.17520119364*i";
+	QTest::newRow("cos complex") << "cos(5-9*i)" << "1149.26926545-3885.12187972*i";
+	QTest::newRow("complex*complex") << "(5.3-9.8*i)*(-6.2+3.7*i)" << "3.4+80.37*i";
+	QTest::newRow("simple complex/complex") << "i/i" << "1";
+	QTest::newRow("complex/complex") << "(9.3-5.4*i)/(3.6-9.5*i)" << "0.82143203178+0.667667861641*i";
+	QTest::newRow("simple complex conjugate") << "conjugate(i)" << "-i";
+	QTest::newRow("complex conjugate") << "conjugate(-9.3+5.87*i)" << "-9.3-5.87*i";
+	QTest::newRow("complex arg") << "arg(i)" << "1.57079632679";
+	QTest::newRow("complex real part") << "real(45-9*i)" << "45";
+	QTest::newRow("complex imag part") << "imaginary(45-9*i)" << "-9";
+	QTest::newRow("simply complex mod") << "abs(i)" << "1";
+	QTest::newRow("complex mod") << "abs(8-9*i)" << "12.0415945788";
 	QTest::newRow("simple addition") << "2+2" << "4";
 	QTest::newRow("simple addition with var") << "2+x" << "x+2";
 	QTest::newRow("minus irreductibility") << "-x" << "-x";
@@ -204,8 +233,25 @@ void AnalitzaTest::testTrivialEvaluate_data()
 	
 	QTest::newRow("matrix") << "matrix { matrixrow { 1, 2 } }" << "matrix { matrixrow { 1, 2 } }";
 	QTest::newRow("matrix+") << "matrix { matrixrow { 1, 2 } }+matrix{ matrixrow { 1, 2 } }" << "matrix { matrixrow { 2, 4 } }";
+	QTest::newRow("matrix++") << "matrix { matrixrow { 5, 6 }, matrixrow { 4, 0 }}+matrix { matrixrow { 2, 3 }, matrixrow { 4, 0 }}" << "matrix { matrixrow { 7, 9 }, matrixrow { 8, 0 } }";
+	//TODO aucahuasi: we support only matrix and vector over a scalar field (numbers), but I think we could have matrix/vector over other structures too (e.g. functions, vectors, etc.)
+	//QTest::newRow("matrix+++") << "matrix { matrixrow { vector { 1, 2 } } }+matrix { matrixrow { vector { 1.8, 2.4 } } }" << "matrix { matrixrow { vector { 2.8, 4.4 } } }";
 	QTest::newRow("matrix@") << "selector(1, matrix { matrixrow { 1, 2 } })" << "vector { 1, 2 }";
 	QTest::newRow("matrix@@") << "selector(1, selector(1, matrix { matrixrow { 1, 2 } }))" << "1";
+	QTest::newRow("scalar multiplication of matrix") << "3*matrix { matrixrow { 5, 6 }, matrixrow { 4, 0 }}" << "matrix { matrixrow { 15, 18 }, matrixrow { 12, 0 } }";
+	QTest::newRow("transpose vector") << "transpose(vector{12,45})" << "matrix { matrixrow { 12, 45 } }";
+	QTest::newRow("row x column") << "matrix { matrixrow { 1, 2 } }*matrix { matrixrow { 3 }, matrixrow { 5 } }" << "matrix { matrixrow { 13 } }";
+	QTest::newRow("column x row") << "matrix { matrixrow { 3 }, matrixrow { 5 } }*matrix { matrixrow { 1, 2 } }" << "matrix { matrixrow { 3, 6 }, matrixrow { 5, 10 } }";
+	QTest::newRow("row x vector") << "matrix { matrixrow { 1, 2 } }*vector{ 3, 5 }" << "vector { 13 }";
+	QTest::newRow("vector x row") << "vector{ 3, 5 }*matrix { matrixrow { 1, 2 } }" << "matrix { matrixrow { 3, 6 }, matrixrow { 5, 10 } }";
+	QTest::newRow("vector x transpose(vector)") << "vector{ 3, 5 }*transpose(vector{1,2})" << "matrix { matrixrow { 3, 6 }, matrixrow { 5, 10 } }";
+	QTest::newRow("matrix x vector") << "matrix { matrixrow { 3, 3 }, matrixrow { 2, 2 }, matrixrow { 3, 4 } }*vector{ 1, 2 }" << "vector { 9, 6, 11 }";
+	QTest::newRow("matrix x matrix") << "matrix { matrixrow { 3, 3 }, matrixrow { 2, 2 }, matrixrow { 3, 4 } }*matrix { matrixrow{3, 3, 4, 5, 6}, matrixrow{2, 4, 5, 6, 2} }" << "matrix { matrixrow { 15, 21, 27, 33, 24 }, matrixrow { 10, 14, 18, 22, 16 }, matrixrow { 17, 25, 32, 39, 26 } }";
+	QTest::newRow("matrix^0") << "power(matrix { matrixrow { 3, 3 }, matrixrow { 2, 2 } }, 0)" << "matrix { matrixrow { 1, 0 }, matrixrow { 0, 1 } }";
+	QTest::newRow("matrix^1") << "power(matrix { matrixrow { 3, 3 }, matrixrow { 2, 2 } }, 1)" << "matrix { matrixrow { 3, 3 }, matrixrow { 2, 2 } }";
+	QTest::newRow("matrix^2") << "power(matrix { matrixrow { 3, 3 }, matrixrow { 2, 2 } }, 2)" << "matrix { matrixrow { 15, 15 }, matrixrow { 10, 10 } }";
+	QTest::newRow("matrix^64") << "power(matrix { matrixrow { 1.63, 2.4}, matrixrow { -0.36,7.128 } }, 64)" << "matrix { matrixrow { -2.79721542669e+52, 4.14615974023e+53 }, matrixrow { -6.21923961035e+52, 9.21843939558e+53 } }";
+	QTest::newRow("matrix^6464") << "power(matrix { matrixrow { 1.0019 } }, 6464)"  << "matrix { matrixrow { 213191.74219 } }";
 }
 
 void AnalitzaTest::testTrivialEvaluate()
@@ -302,6 +348,12 @@ void AnalitzaTest::testCorrection_data()
 	QTest::addColumn<QString>("result");
 	
 	QStringList script;
+	
+	script.clear();
+	script << "f:=y->y*y";
+	script << "f(i)";
+	QTest::newRow("from complex function") << script << "-1";
+	
 	script.clear();
 	script << "n:=2";
 	script << "n+1";
@@ -609,8 +661,8 @@ void AnalitzaTest::testSimplify_data()
 	QTest::newRow("diff") << "diff(x^2:x)" << "x->2*x";
 	QTest::newRow("sum times") << "sum(n*x : n=0..99)" << "4950*x";
 	QTest::newRow("levelout") << "-y-(x+y)" << "-2*y-x";
-	QTest::newRow("sum") << "n->sum((i+n) * i : i=0..9)" << "n->sum((i+n)*i:i=0..9)";
-	QTest::newRow("sum.sum") << "k->sum(sum(x:x=0..i):i=0..k)" << "k->sum(sum(x:x=0..i):i=0..k)";
+	QTest::newRow("sum") << "n->sum((s+n) * s : s=0..9)" << "n->sum((s+n)*s:s=0..9)";
+	QTest::newRow("sum.sum") << "k->sum(sum(x:x=0..s):s=0..k)" << "k->sum(sum(x:x=0..s):s=0..k)";
 	QTest::newRow("unrelated sum") << "sum(x : n=0..99)" << "100*x";
 	QTest::newRow("ln") << "ln(x)" << "ln(x)";
 	
@@ -748,6 +800,7 @@ void AnalitzaTest::testCrash_data()
 	QTest::newRow("wrong func") << "xsin(x)";
 	QTest::newRow("scalarprod") << "scalarproduct(vector{0}, vector{x,0})";
 	QTest::newRow("power") << "list{}**2";
+	QTest::newRow("unary-nested-error") << "-(2/0)";
 }
 
 void AnalitzaTest::testCrash()
@@ -893,4 +946,4 @@ void AnalitzaTest::testOperators()
 	}
 }
 
-#include "analitzatest.moc"
+
