@@ -21,7 +21,6 @@
 #include "private/functiongraphfactory.h"
 #include <private/utils/mathutils.h>
 
-#include <analitza/localize.h>
 #include <analitza/value.h>
 #include <analitza/vector.h>
 
@@ -33,7 +32,7 @@ public:
     explicit SphericalSurface(const Analitza::Expression& e);
     SphericalSurface(const Analitza::Expression& e, Analitza::Variables* v);
     
-    TYPE_NAME(I18N_NOOP("Spherical Surface Radial=F(t:Azimuth, p: Polar)"))
+    TYPE_NAME(QT_TR_NOOP("Spherical Surface Radial=F(t:Azimuth, p: Polar)"))
     EXPRESSION_TYPE(Analitza::ExpressionType(Analitza::ExpressionType::Lambda).addParameter(
                         Analitza::ExpressionType(Analitza::ExpressionType::Value)).addParameter(
                         Analitza::ExpressionType(Analitza::ExpressionType::Value)).addParameter(
